"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@checkup/core");
const chalk = require("chalk");
class EslintSummaryTaskResult extends core_1.BaseTaskResult {
    constructor() {
        super(...arguments);
        this.actions = [];
    }
    process(data) {
        this.data = Object.assign({ sortedESLintReport: transformEslintReport(data.esLintReport) }, data);
        let actionsEvaluator = new core_1.ActionsEvaluator();
        let errorCount = this.data.sortedESLintReport.errorCount;
        let warningCount = this.data.sortedESLintReport.warningCount;
        actionsEvaluator.add({
            name: 'reduce-eslint-errors',
            summary: 'Reduce number of eslint errors',
            details: `${errorCount} total errors`,
            defaultThreshold: 20,
            items: [`Total eslint errors: ${errorCount}`],
            input: errorCount,
        });
        actionsEvaluator.add({
            name: 'reduce-eslint-warnings',
            summary: 'Reduce number of eslint warnings',
            details: `${warningCount} total warnings`,
            defaultThreshold: 20,
            items: [`Total eslint warnings: ${warningCount}`],
            input: warningCount,
        });
        this.actions = actionsEvaluator.evaluate(this.config);
    }
    toConsole() {
        core_1.ui.section(this.meta.friendlyTaskName, () => {
            core_1.ui.log(`Error count: ${chalk.red(this.data.sortedESLintReport.errorCount)}`);
            core_1.ui.log(`Warning count: ${chalk.yellow(this.data.sortedESLintReport.warningCount)}`);
            if (this.data.sortedESLintReport.errorList.length > 0) {
                core_1.ui.blankLine();
                core_1.ui.subHeader('Errors');
                core_1.ui.table(this.data.sortedESLintReport.errorList, {
                    rule: { header: 'Rule name' },
                    failures: {},
                });
            }
            if (this.data.sortedESLintReport.warningList.length > 0) {
                core_1.ui.blankLine();
                core_1.ui.subHeader('Warnings');
                core_1.ui.table(this.data.sortedESLintReport.warningList, {
                    rule: { header: 'Rule name' },
                    failures: {},
                });
            }
        });
    }
    toJson() {
        return { meta: this.meta, result: { esLintReport: this.data.esLintReport } };
    }
}
exports.default = EslintSummaryTaskResult;
function transformEslintReport(esLintReport) {
    let errors = {};
    let warnings = {};
    // we want to strip these from total counts (generally they are just a warning that a file was ignored)
    let warningsWithoutRulesIds = 0;
    esLintReport.results.forEach((result) => {
        if (result.messages.length > 0) {
            result.messages.forEach((msg) => {
                // severity 1 is a warning
                if (msg.severity === 1) {
                    if (msg.ruleId) {
                        updateFailureCounts(msg, warnings);
                    }
                    else {
                        warningsWithoutRulesIds++;
                    }
                }
                else {
                    updateFailureCounts(msg, errors);
                }
            });
        }
    });
    return {
        errorCount: esLintReport.errorCount,
        warningCount: esLintReport.warningCount - warningsWithoutRulesIds,
        errorList: createFailureList(errors, 'errors'),
        warningList: createFailureList(warnings, 'warnings'),
    };
}
function updateFailureCounts(msg, record) {
    if (msg.ruleId) {
        // this is the first time we've encountered this rule being violated, so create a record of it
        if (!record[msg.ruleId]) {
            record[msg.ruleId] = {
                fixableFailureCount: 0,
                failureCount: 0,
            };
        }
        if (msg.fix) {
            record[msg.ruleId].fixableFailureCount += 1;
        }
        else {
            record[msg.ruleId].failureCount += 1;
        }
    }
}
function createFailureList(record, failureType) {
    let chalkColor = failureType === 'errors' ? chalk.red : chalk.yellow;
    return Object.entries(record).map((arr) => {
        let ruleId = arr[0];
        let eslintFailureCounts = arr[1];
        let totalFailureCount = eslintFailureCounts.failureCount + eslintFailureCounts.fixableFailureCount;
        return {
            rule: ruleId,
            failures: `${chalkColor(totalFailureCount)} ${failureType} (${eslintFailureCounts.fixableFailureCount} fixable)`,
        };
    });
}
//# sourceMappingURL=eslint-summary-task-result.js.map